@@ -12,9 +12,7 @@
 
 #include "enums/messagecomponenttype.h"
 #include "models/customemojimodel.h"
-#include "models/messagecontentmodel.h"
 #include "neochatconnection.h"
-#include "utils.h"
 
 #include "testutils.h"
 
@@ -46,6 +44,7 @@ private Q_SLOTS:
     void sendCustomEmojiCode_data();
     void sendCustomEmojiCode();
 
+    void receiveSpacelessSelfClosingTag();
     void receiveStripReply();
     void receivePlainTextIn();
 
@@ -85,11 +84,11 @@ void TextHandlerTest::initTestCase()
 
 void TextHandlerTest::allowedAttributes()
 {
-    const QString testInputString1 = QStringLiteral("<p><span data-mx-spoiler><font color=#FFFFFF>Test</font><span></p>");
-    const QString testOutputString1 = QStringLiteral("<p><span data-mx-spoiler><font color=#FFFFFF>Test</font><span></p>");
+    const QString testInputString1 = QStringLiteral("<span data-mx-spoiler><font color=#FFFFFF>Test</font><span>");
+    const QString testOutputString1 = QStringLiteral("<span data-mx-spoiler><font color=#FFFFFF>Test</font><span>");
     // Handle urls where the href has either single (') or double (") quotes.
-    const QString testInputString2 = QStringLiteral("<p><a href=\"https://kde.org\">link</a><a href='https://kde.org'>link</a></p>");
-    const QString testOutputString2 = QStringLiteral("<p><a href=\"https://kde.org\">link</a><a href='https://kde.org'>link</a></p>");
+    const QString testInputString2 = QStringLiteral("<a href=\"https://kde.org\">link</a><a href='https://kde.org'>link</a>");
+    const QString testOutputString2 = QStringLiteral("<a href=\"https://kde.org\">link</a><a href='https://kde.org'>link</a>");
 
     TextHandler testTextHandler;
     testTextHandler.setData(testInputString1);
@@ -117,7 +116,7 @@ void TextHandlerTest::stripDisallowedTags()
 void TextHandlerTest::stripDisallowedAttributes()
 {
     const QString testInputString = QStringLiteral("<p style=\"font-size:50px;\" color=#FFFFFF>Test</p>");
-    const QString testOutputString = QStringLiteral("<p>Test</p>");
+    const QString testOutputString = QStringLiteral("Test");
 
     TextHandler testTextHandler;
     testTextHandler.setData(testInputString);
@@ -144,8 +143,8 @@ void TextHandlerTest::emptyCodeTags()
 
 void TextHandlerTest::sendSimpleStringCase()
 {
-    const QString testInputString = QStringLiteral("This data should just be put in a paragraph.");
-    const QString testOutputString = QStringLiteral("<p>This data should just be put in a paragraph.</p>");
+    const QString testInputString = QStringLiteral("This data should just be left alone.");
+    const QString testOutputString = QStringLiteral("This data should just be left alone.");
 
     TextHandler testTextHandler;
     testTextHandler.setData(testInputString);
@@ -158,8 +157,8 @@ void TextHandlerTest::sendSingleParaMarkup()
     const QString testInputString = QStringLiteral(
         "Text para with **bold**, *italic*, [link](https://kde.org), ![image](mxc://kde.org/aebd3ffd40503e1ef0525bf8f0d60282fec6183e), `inline code`.");
     const QString testOutputString = QStringLiteral(
-        "<p>Text para with <strong>bold</strong>, <em>italic</em>, <a href=\"https://kde.org\">link</a>, <img "
-        "src=\"mxc://kde.org/aebd3ffd40503e1ef0525bf8f0d60282fec6183e\" alt=\"image\">, <code>inline code</code>.</p>");
+        "Text para with <strong>bold</strong>, <em>italic</em>, <a href=\"https://kde.org\">link</a>, <img "
+        "src=\"mxc://kde.org/aebd3ffd40503e1ef0525bf8f0d60282fec6183e\" alt=\"image\">, <code>inline code</code>.");
 
     TextHandler testTextHandler;
     testTextHandler.setData(testInputString);
@@ -185,7 +184,7 @@ void TextHandlerTest::sendMultipleSectionMarkup()
 void TextHandlerTest::sendBadLinks()
 {
     const QString testInputString = QStringLiteral("[link](kde.org), ![image](https://kde.org/aebd3ffd40503e1ef0525bf8f0d60282fec6183e)");
-    const QString testOutputString = QStringLiteral("<p><a>link</a>, <img alt=\"image\"></p>");
+    const QString testOutputString = QStringLiteral("<a>link</a>, <img alt=\"image\">");
 
     TextHandler testTextHandler;
     testTextHandler.setData(testInputString);
@@ -222,8 +221,8 @@ void TextHandlerTest::sendCodeClass()
 void TextHandlerTest::sendCustomEmoji()
 {
     const QString testInputString = QStringLiteral(":test:");
-    const QString testOutputString = QStringLiteral(
-        "<p><img data-mx-emoticon=\"\" src=\"mxc://example.org/test\" alt=\":test:\" title=\":test:\" height=\"32\" vertical-align=\"middle\" /></p>");
+    const QString testOutputString =
+        QStringLiteral("<img data-mx-emoticon=\"\" src=\"mxc://example.org/test\" alt=\":test:\" title=\":test:\" height=\"32\" vertical-align=\"middle\" />");
 
     TextHandler testTextHandler;
     testTextHandler.setData(testInputString);
@@ -236,7 +235,7 @@ void TextHandlerTest::sendCustomEmojiCode_data()
     QTest::addColumn<QString>("testInputString");
     QTest::addColumn<QString>("testOutputString");
 
-    QTest::newRow("inline") << QStringLiteral("`:test:`") << QStringLiteral("<p><code>:test:</code></p>");
+    QTest::newRow("inline") << QStringLiteral("`:test:`") << QStringLiteral("<code>:test:</code>");
     QTest::newRow("block") << QStringLiteral("```\n:test:\n```") << QStringLiteral("<pre><code>:test:\n</code></pre>");
 }
 
@@ -252,6 +251,19 @@ void TextHandlerTest::sendCustomEmojiCode()
     QCOMPARE(testTextHandler.handleSendText(), testOutputString);
 }
 
+void TextHandlerTest::receiveSpacelessSelfClosingTag()
+{
+    const QString testInputString = QStringLiteral("Test...<br/>...ing");
+    const QString testRichOutputString = QStringLiteral("Test...<br/>...ing");
+    const QString testPlainOutputString = QStringLiteral("Test...\n...ing");
+
+    TextHandler testTextHandler;
+    testTextHandler.setData(testInputString);
+
+    QCOMPARE(testTextHandler.handleRecieveRichText(), testRichOutputString);
+    QCOMPARE(testTextHandler.handleRecievePlainText(Qt::RichText), testPlainOutputString);
+}
+
 void TextHandlerTest::receiveStripReply()
 {
     const QString testInputString = QStringLiteral(
@@ -274,6 +286,7 @@ void TextHandlerTest::receiveRichInPlainOut_data()
     QTest::newRow("ampersand") << QStringLiteral("a &amp; b") << QStringLiteral("a & b");
     QTest::newRow("quote") << QStringLiteral("&quot;a and b&quot;") << QStringLiteral("\"a and b\"");
     QTest::newRow("new line") << QStringLiteral("new<br>line") << QStringLiteral("new\nline");
+    QTest::newRow("unescape") << QStringLiteral("can&#x27;t") << QStringLiteral("can't");
 }
 
 void TextHandlerTest::receiveRichInPlainOut()
@@ -360,7 +373,7 @@ void TextHandlerTest::receivePlainStripMarkup()
 void TextHandlerTest::receiveRichUserPill()
 {
     const QString testInputString = QStringLiteral("<p><a href=\"https://matrix.to/#/@alice:example.org\">@alice:example.org</a></p>");
-    const QString testOutputString = QStringLiteral("<p><b><a href=\"https://matrix.to/#/@alice:example.org\">@alice:example.org</a></b></p>");
+    const QString testOutputString = QStringLiteral("<b><a href=\"https://matrix.to/#/@alice:example.org\">@alice:example.org</a></b>");
 
     TextHandler testTextHandler;
     testTextHandler.setData(testInputString);
@@ -371,7 +384,7 @@ void TextHandlerTest::receiveRichUserPill()
 void TextHandlerTest::receiveRichStrikethrough()
 {
     const QString testInputString = QStringLiteral("<p><del>Test</del></p>");
-    const QString testOutputString = QStringLiteral("<p><s>Test</s></p>");
+    const QString testOutputString = QStringLiteral("<s>Test</s>");
 
     TextHandler testTextHandler;
     testTextHandler.setData(testInputString);
@@ -448,6 +461,9 @@ void TextHandlerTest::receiveRichPlainUrl()
     QString testOutputStringMxId = QStringLiteral(
         "<b><a href=\"https://matrix.to/#/@user:kde.org\">@user:kde.org</a></b> <b><a href=\"https://matrix.to/#/@user:kde.org\">Link already rich</a></b>");
 
+    QString testInputStringMxIdWithPrefix = QStringLiteral("a @user:kde.org b");
+    QString testOutputStringMxIdWithPrefix = QStringLiteral("a <b><a href=\"https://matrix.to/#/@user:kde.org\">@user:kde.org</a></b> b");
+
     TextHandler testTextHandler;
     testTextHandler.setData(testInputStringLink1);
 
@@ -461,6 +477,9 @@ void TextHandlerTest::receiveRichPlainUrl()
 
     testTextHandler.setData(testInputStringMxId);
     QCOMPARE(testTextHandler.handleRecieveRichText(Qt::RichText), testOutputStringMxId);
+
+    testTextHandler.setData(testInputStringMxIdWithPrefix);
+    QCOMPARE(testTextHandler.handleRecieveRichText(Qt::RichText), testOutputStringMxIdWithPrefix);
 }
 
 void TextHandlerTest::receiveRichEdited_data()
@@ -535,7 +554,7 @@ void TextHandlerTest::componentOutput_data()
                "someField }\nCustomQml {\n    someTextProperty: someField.text\n}\n</code></pre>Sure you can, it's still local to the same file where you "
                "defined the id")
         << QList<MessageComponent>{
-               MessageComponent{MessageComponentType::Text, QStringLiteral("Ah, you mean something like"), {}},
+               MessageComponent{MessageComponentType::Text, QStringLiteral("Ah, you mean something like<br/>"), {}},
                MessageComponent{
                    MessageComponentType::Code,
                    QStringLiteral(
