@@ -13,7 +13,6 @@
 
 #include <signal.h>
 
-#include <Quotient/accountregistry.h>
 #include <Quotient/csapi/notifications.h>
 #include <Quotient/qt_connection_util.h>
 #include <Quotient/settings.h>
@@ -64,11 +63,7 @@ Controller::Controller(QObject *parent)
         });
     } else {
         auto c = new NeoChatConnection(this);
-#if Quotient_VERSION_MINOR > 8
         c->assumeIdentity(QStringLiteral("@user:localhost:1234"), QStringLiteral("device_1234"), QStringLiteral("token_1234"));
-#else
-        c->assumeIdentity(QStringLiteral("@user:localhost:1234"), QStringLiteral("token_1234"));
-#endif
         connect(c, &Connection::connected, this, [c, this]() {
             m_accountRegistry.add(c);
             c->syncLoop();
@@ -170,6 +165,10 @@ void Controller::addConnection(NeoChatConnection *c)
         dropConnection(c);
     });
     connect(c, &NeoChatConnection::badgeNotificationCountChanged, this, &Controller::updateBadgeNotificationCount);
+    connect(c, &NeoChatConnection::syncDone, this, [this, c]() {
+        m_notificationsManager.handleNotifications(c);
+    });
+    connect(c, &NeoChatConnection::showInviteNotification, &m_notificationsManager, &NotificationsManager::postInviteNotification);
 
     c->sync();
 
@@ -180,6 +179,8 @@ void Controller::dropConnection(NeoChatConnection *c)
 {
     Q_ASSERT_X(c, __FUNCTION__, "Attempt to drop a null connection");
 
+    c->disconnect(this);
+    c->disconnect(&m_notificationsManager);
     m_accountRegistry.drop(c);
     Q_EMIT connectionDropped(c);
 }
@@ -225,14 +226,7 @@ void Controller::invokeLogin()
                             Qt::SingleShotConnection);
                     }
                 });
-                connect(connection, &NeoChatConnection::networkError, this, [this](const QString &error, const QString &, int, int) {
-                    Q_EMIT errorOccured(i18n("Network Error: %1", error), {});
-                });
-#if Quotient_VERSION_MINOR > 8
                 connection->assumeIdentity(account.userId(), account.deviceId(), accessToken);
-#else
-                connection->assumeIdentity(account.userId(), accessToken);
-#endif
             });
         }
     }
@@ -252,17 +246,17 @@ QKeychain::ReadPasswordJob *Controller::loadAccessTokenFromKeyChain(const QStrin
 
         switch (job->error()) {
         case QKeychain::EntryNotFound:
-            Q_EMIT errorOccured(i18n("Access token wasn't found"), i18n("Maybe it was deleted?"));
+            Q_EMIT errorOccured(i18n("Access token wasn't found: Maybe it was deleted?"));
             break;
         case QKeychain::AccessDeniedByUser:
         case QKeychain::AccessDenied:
-            Q_EMIT errorOccured(i18n("Access to keychain was denied."), i18n("Please allow NeoChat to read the access token"));
+            Q_EMIT errorOccured(i18n("Access to keychain was denied: Please allow NeoChat to read the access token"));
             break;
         case QKeychain::NoBackendAvailable:
-            Q_EMIT errorOccured(i18n("No keychain available."), i18n("Please install a keychain, e.g. KWallet or GNOME keyring on Linux"));
+            Q_EMIT errorOccured(i18n("No keychain available: Please install a keychain, e.g. KWallet or GNOME keyring on Linux"));
             break;
         case QKeychain::OtherError:
-            Q_EMIT errorOccured(i18n("Unable to read access token"), job->errorString());
+            Q_EMIT errorOccured(i18n("Unable to read access token: %1", job->errorString()));
             break;
         default:
             break;
@@ -327,11 +321,18 @@ void Controller::setActiveConnection(NeoChatConnection *connection)
         return;
     }
 
+    if (m_connection != nullptr) {
+        m_connection->disconnect(this);
+        m_connection->disconnect(&m_notificationsManager);
+    }
+
     m_connection = connection;
 
     if (m_connection != nullptr) {
         m_connection->refreshBadgeNotificationCount();
         updateBadgeNotificationCount(m_connection, m_connection->badgeNotificationCount());
+
+        connect(m_connection, &NeoChatConnection::errorOccured, this, &Controller::errorOccured);
     }
 
     Q_EMIT activeConnectionChanged(m_connection);
@@ -346,7 +347,7 @@ void Controller::listenForNotifications()
     connect(timer, &QTimer::timeout, qGuiApp, &QGuiApplication::quit);
 
     connect(connector, &KUnifiedPush::Connector::messageReceived, [timer](const QByteArray &data) {
-        NotificationsManager::instance().postPushNotification(data);
+        instance().m_notificationsManager.postPushNotification(data);
         timer->stop();
     });
 
@@ -358,6 +359,11 @@ void Controller::listenForNotifications()
 #endif
 }
 
+void Controller::clearInvitationNotification(const QString &roomId)
+{
+    m_notificationsManager.clearInvitationNotification(roomId);
+}
+
 void Controller::updateBadgeNotificationCount(NeoChatConnection *connection, int count)
 {
     if (connection == m_connection) {
@@ -427,11 +433,24 @@ void Controller::removeConnection(const QString &userId)
 
 bool Controller::csSupported() const
 {
-#if Quotient_VERSION_MINOR > 9
     return true;
-#else
-    return false;
-#endif
+}
+
+void Controller::revertToDefaultConfig()
+{
+    const auto config = NeoChatConfig::self();
+    config->setDefaults();
+    config->save();
+}
+
+bool Controller::isImageShown(const QString &eventId)
+{
+    return m_shownImages.contains(eventId);
+}
+
+void Controller::markImageShown(const QString &eventId)
+{
+    m_shownImages.append(eventId);
 }
 
 #include "moc_controller.cpp"
