@@ -48,7 +48,6 @@
 #include "colorschemer.h"
 #include "controller.h"
 #include "logger.h"
-#include "neochatconfig.h"
 #include "roommanager.h"
 #include "sharehandler.h"
 #include "windowcontroller.h"
@@ -141,14 +140,14 @@ int main(int argc, char *argv[])
     KAboutData about(QStringLiteral("neochat"),
                      i18n("NeoChat"),
                      QStringLiteral(NEOCHAT_VERSION_STRING),
-                     i18n("Matrix client"),
+                     i18n("Chat on Matrix"),
                      KAboutLicense::GPL_V3,
                      i18n("© 2018-2020 Black Hat, 2020-2024 KDE Community"));
     about.addAuthor(i18n("Carl Schwan"),
                     i18n("Maintainer"),
                     QStringLiteral("carl@carlschwan.eu"),
                     QStringLiteral("https://carlschwan.eu"),
-                    QStringLiteral("https://carlschwan.eu/avatar.png"));
+                    QUrl(QStringLiteral("https://carlschwan.eu/avatar.png")));
     about.addAuthor(i18n("Tobias Fella"), i18n("Maintainer"), QStringLiteral("tobias.fella@kde.org"), QStringLiteral("https://tobiasfella.de"));
     about.addAuthor(i18n("James Graham"), i18n("Maintainer"), QStringLiteral("james.h.graham@protonmail.com"));
     about.addCredit(i18n("Black Hat"), i18n("Original author of Spectral"), QStringLiteral("bhat@encom.eu.org"));
@@ -185,9 +184,6 @@ int main(int argc, char *argv[])
 #endif
 
     ColorSchemer colorScheme;
-    if (!NeoChatConfig::self()->colorScheme().isEmpty()) {
-        colorScheme.apply(NeoChatConfig::self()->colorScheme());
-    }
 
     QCommandLineParser parser;
     parser.setApplicationDescription(i18n("Client for the matrix communication protocol"));
@@ -307,7 +303,6 @@ int main(int argc, char *argv[])
     QWindow *window = windowFromEngine(&engine);
 
     WindowController::instance().setWindow(window);
-    WindowController::instance().restoreGeometry();
 
     return app.exec();
 }
