@@ -2,8 +2,8 @@
 // SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 #include "messagecontentmodel.h"
+#include "eventhandler.h"
 #include "neochatconfig.h"
-#include "neochatroommember.h"
 
 #include <QImageReader>
 
@@ -11,9 +11,10 @@
 #include <Quotient/events/redactionevent.h>
 #include <Quotient/events/roommessageevent.h>
 #include <Quotient/events/stickerevent.h>
+#include <Quotient/qt_connection_util.h>
 
 #include <KLocalizedString>
-#include <Quotient/qt_connection_util.h>
+#include <Kirigami/Platform/PlatformTheme>
 
 #ifndef Q_OS_ANDROID
 #include <KSyntaxHighlighting/Definition>
@@ -21,10 +22,7 @@
 #endif
 
 #include "chatbarcache.h"
-#include "enums/messagecomponenttype.h"
-#include "eventhandler.h"
 #include "filetype.h"
-#include "itinerarymodel.h"
 #include "linkpreviewer.h"
 #include "neochatconnection.h"
 #include "neochatroom.h"
@@ -32,20 +30,8 @@
 
 using namespace Quotient;
 
-MessageContentModel::MessageContentModel(NeoChatRoom *room, const Quotient::RoomEvent *event, bool isReply, bool isPending)
-    : QAbstractListModel(nullptr)
-    , m_room(room)
-    , m_eventId(event != nullptr ? event->id() : QString())
-    , m_eventSenderId(event != nullptr ? event->senderId() : QString())
-    , m_isPending(isPending)
-    , m_isReply(isReply)
-{
-    intiializeEvent(event);
-    initializeModel();
-}
-
-MessageContentModel::MessageContentModel(NeoChatRoom *room, const QString &eventId, bool isReply, bool isPending)
-    : QAbstractListModel(nullptr)
+MessageContentModel::MessageContentModel(NeoChatRoom *room, const QString &eventId, bool isReply, bool isPending, MessageContentModel *parent)
+    : QAbstractListModel(parent)
     , m_room(room)
     , m_eventId(eventId)
     , m_isPending(isPending)
@@ -57,76 +43,79 @@ MessageContentModel::MessageContentModel(NeoChatRoom *room, const QString &event
 void MessageContentModel::initializeModel()
 {
     Q_ASSERT(m_room != nullptr);
-    // Allow making a model for an event that is being downloaded but will appear later
-    // e.g. a reply, but we need an ID to know when it has arrived.
     Q_ASSERT(!m_eventId.isEmpty());
 
-    Quotient::connectUntil(m_room.get(), &NeoChatRoom::extraEventLoaded, this, [this](const QString &eventId) {
-        if (m_room != nullptr) {
-            if (eventId == m_eventId) {
-                m_event = loadEvent<RoomEvent>(m_room->getEvent(eventId)->fullJson());
-                Q_EMIT eventUpdated();
-                updateReplyModel();
-                resetContent();
-                return true;
-            }
-        }
-        return false;
-    });
-
-    if (m_event == nullptr) {
-        m_room->downloadEventFromServer(m_eventId);
-    }
+    connect(this, &MessageContentModel::eventUnavailable, this, &MessageContentModel::getEvent);
 
     connect(m_room, &NeoChatRoom::pendingEventAboutToMerge, this, [this](Quotient::RoomEvent *serverEvent) {
-        if (m_room != nullptr && m_event != nullptr) {
-            if (m_eventId == serverEvent->id()) {
+        if (m_room != nullptr) {
+            if (m_eventId == serverEvent->id() || m_eventId == serverEvent->transactionId()) {
                 beginResetModel();
                 m_isPending = false;
-                intiializeEvent(serverEvent);
+                m_eventId = serverEvent->id();
+                initializeEvent();
                 endResetModel();
             }
         }
     });
+    connect(m_room, &NeoChatRoom::addedMessages, this, [this](int fromIndex, int toIndex) {
+        if (m_room != nullptr) {
+            for (int i = fromIndex; i <= toIndex; i++) {
+                if (m_room->findInTimeline(i)->event()->id() == m_eventId) {
+                    initializeEvent();
+                    updateReplyModel();
+                    resetModel();
+                }
+            }
+        }
+    });
     connect(m_room, &NeoChatRoom::replacedEvent, this, [this](const Quotient::RoomEvent *newEvent) {
-        if (m_room != nullptr && m_event != nullptr) {
+        if (m_room != nullptr) {
             if (m_eventId == newEvent->id()) {
                 beginResetModel();
-                intiializeEvent(newEvent);
+                initializeEvent();
+                resetContent();
                 endResetModel();
             }
         }
     });
     connect(m_room, &NeoChatRoom::newFileTransfer, this, [this](const QString &eventId) {
-        if (m_event != nullptr && eventId == m_eventId) {
+        if (eventId == m_eventId) {
             Q_EMIT dataChanged(index(0), index(rowCount() - 1), {FileTransferInfoRole});
         }
     });
     connect(m_room, &NeoChatRoom::fileTransferProgress, this, [this](const QString &eventId) {
-        if (m_event != nullptr && eventId == m_eventId) {
+        if (eventId == m_eventId) {
             Q_EMIT dataChanged(index(0), index(rowCount() - 1), {FileTransferInfoRole});
         }
     });
     connect(m_room, &NeoChatRoom::fileTransferCompleted, this, [this](const QString &eventId) {
-        if (m_room != nullptr && m_event != nullptr && eventId == m_eventId) {
+        if (m_room != nullptr && eventId == m_eventId) {
             resetContent();
             Q_EMIT dataChanged(index(0), index(rowCount() - 1), {FileTransferInfoRole});
         }
     });
     connect(m_room, &NeoChatRoom::fileTransferFailed, this, [this](const QString &eventId) {
-        if (m_event != nullptr && eventId == m_eventId) {
+        if (eventId == m_eventId) {
             resetContent();
             Q_EMIT dataChanged(index(0), index(rowCount() - 1), {FileTransferInfoRole});
         }
     });
     connect(m_room->editCache(), &ChatBarCache::relationIdChanged, this, [this](const QString &oldEventId, const QString &newEventId) {
-        if (m_event != nullptr && (oldEventId == m_eventId || newEventId == m_eventId)) {
+        if (oldEventId == m_eventId || newEventId == m_eventId) {
             // HACK: Because DelegateChooser can't switch the delegate on dataChanged it has to think there is a new delegate.
             beginResetModel();
             resetContent(newEventId == m_eventId);
             endResetModel();
         }
     });
+    connect(m_room->threadCache(), &ChatBarCache::threadIdChanged, this, [this](const QString &oldThreadId, const QString &newThreadId) {
+        if (oldThreadId == m_eventId || newThreadId == m_eventId) {
+            beginResetModel();
+            resetContent(false, newThreadId == m_eventId);
+            endResetModel();
+        }
+    });
     connect(m_room, &NeoChatRoom::urlPreviewEnabledChanged, this, [this]() {
         resetContent();
     });
@@ -134,49 +123,78 @@ void MessageContentModel::initializeModel()
         resetContent();
     });
     connect(m_room, &Room::memberNameUpdated, this, [this](RoomMember member) {
-        if (m_room != nullptr && m_event != nullptr) {
+        if (m_room != nullptr) {
             if (m_eventSenderId.isEmpty() || m_eventSenderId == member.id()) {
                 Q_EMIT dataChanged(index(0, 0), index(rowCount() - 1, 0), {AuthorRole});
             }
         }
     });
     connect(m_room, &Room::memberAvatarUpdated, this, [this](RoomMember member) {
-        if (m_room != nullptr && m_event != nullptr) {
+        if (m_room != nullptr) {
             if (m_eventSenderId.isEmpty() || m_eventSenderId == member.id()) {
                 Q_EMIT dataChanged(index(0, 0), index(rowCount() - 1, 0), {AuthorRole});
             }
         }
     });
 
-    if (m_event != nullptr) {
+    connect(NeoChatConfig::self(), &NeoChatConfig::ThreadsChanged, this, [this]() {
         updateReplyModel();
-    }
+        resetModel();
+    });
+
+    initializeEvent();
+    updateReplyModel();
     resetModel();
 }
 
-void MessageContentModel::intiializeEvent(const QString &eventId)
+void MessageContentModel::initializeEvent()
 {
-    const auto newEvent = m_room->getEvent(eventId);
-    if (newEvent != nullptr) {
-        intiializeEvent(newEvent);
+    const auto event = m_room->getEvent(m_eventId);
+    if (event == nullptr) {
+        Q_EMIT eventUnavailable();
+        return;
     }
-}
 
-void MessageContentModel::intiializeEvent(const Quotient::RoomEvent *event)
-{
-    m_event = loadEvent<RoomEvent>(event->fullJson());
-    auto senderId = event->senderId();
-    // A pending event might not have a sender ID set yet but in that case it must
-    // be the local member.
-    if (senderId.isEmpty()) {
-        senderId = m_room->localMember().id();
-    }
     if (m_eventSenderObject == nullptr) {
+        auto senderId = event->senderId();
+        // A pending event might not have a sender ID set yet but in that case it must
+        // be the local member.
+        if (senderId.isEmpty()) {
+            senderId = m_room->localMember().id();
+        }
         m_eventSenderObject = std::unique_ptr<NeochatRoomMember>(new NeochatRoomMember(m_room, senderId));
     }
     Q_EMIT eventUpdated();
 }
 
+void MessageContentModel::getEvent()
+{
+    Quotient::connectUntil(m_room.get(), &NeoChatRoom::extraEventLoaded, this, [this](const QString &eventId) {
+        if (m_room != nullptr) {
+            if (eventId == m_eventId) {
+                m_notFound = false;
+                initializeEvent();
+                updateReplyModel();
+                resetModel();
+                return true;
+            }
+        }
+        return false;
+    });
+    Quotient::connectUntil(m_room.get(), &NeoChatRoom::extraEventNotFound, this, [this](const QString &eventId) {
+        if (m_room != nullptr) {
+            if (eventId == m_eventId) {
+                m_notFound = true;
+                resetModel();
+                return true;
+            }
+        }
+        return false;
+    });
+
+    m_room->downloadEventFromServer(m_eventId);
+}
+
 bool MessageContentModel::showAuthor() const
 {
     return m_showAuthor;
@@ -189,7 +207,7 @@ void MessageContentModel::setShowAuthor(bool showAuthor)
     }
     m_showAuthor = showAuthor;
 
-    if (m_event != nullptr) {
+    if (m_room->connection()->isIgnored(m_eventSenderId)) {
         if (showAuthor) {
             beginInsertRows({}, 0, 0);
             m_components.prepend(MessageComponent{MessageComponentType::Author, QString(), {}});
@@ -216,20 +234,49 @@ QVariant MessageContentModel::data(const QModelIndex &index, int role) const
         return {};
     }
 
-    EventHandler eventHandler(m_room, m_event.get());
     const auto component = m_components[index.row()];
 
-    if (role == DisplayRole) {
-        if (component.type == MessageComponentType::Loading && m_isReply) {
-            return i18n("Loading reply");
+    const auto event = m_room->getEvent(m_eventId);
+    if (event == nullptr) {
+        if (role == DisplayRole) {
+            if (m_isReply) {
+                return i18n("Loading reply");
+            } else {
+                return i18n("Loading");
+            }
         }
-        if (m_event == nullptr) {
-            return QString();
+        if (role == ComponentTypeRole) {
+            return component.type;
+        }
+        return {};
+    }
+
+    if (role == DisplayRole) {
+        if (m_notFound || m_room->connection()->isIgnored(m_eventSenderId)) {
+            Kirigami::Platform::PlatformTheme *theme =
+                static_cast<Kirigami::Platform::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::Platform::PlatformTheme>(this, true));
+
+            QString disabledTextColor;
+            if (theme != nullptr) {
+                disabledTextColor = theme->disabledTextColor().name();
+            } else {
+                disabledTextColor = QStringLiteral("#000000");
+            }
+            return QString(QStringLiteral("<span style=\"color:%1\">").arg(disabledTextColor)
+                           + i18nc("@info", "This message was either not found, you do not have permission to view it, or it was sent by an ignored user")
+                           + QStringLiteral("</span>"));
+        }
+        if (component.type == MessageComponentType::Loading) {
+            if (m_isReply) {
+                return i18n("Loading reply");
+            } else {
+                return i18n("Loading");
+            }
         }
         if (!component.content.isEmpty()) {
             return component.content;
         }
-        return eventHandler.getRichBody();
+        return EventHandler::richBody(m_room, event);
     }
     if (role == ComponentTypeRole) {
         return component.type;
@@ -238,56 +285,53 @@ QVariant MessageContentModel::data(const QModelIndex &index, int role) const
         return component.attributes;
     }
     if (role == EventIdRole) {
-        return eventHandler.getId();
+        return EventHandler::id(event);
     }
     if (role == TimeRole) {
-        const auto pendingIt = std::find_if(m_room->pendingEvents().cbegin(), m_room->pendingEvents().cend(), [this](const PendingEventItem &pendingEvent) {
-            return m_event->transactionId() == pendingEvent->transactionId();
+        const auto pendingIt = std::find_if(m_room->pendingEvents().cbegin(), m_room->pendingEvents().cend(), [event](const PendingEventItem &pendingEvent) {
+            return event->transactionId() == pendingEvent->transactionId();
         });
 
         auto lastUpdated = pendingIt == m_room->pendingEvents().cend() ? QDateTime() : pendingIt->lastUpdated();
-        return eventHandler.getTime(m_isPending, lastUpdated);
+        return EventHandler::time(event, m_isPending, lastUpdated);
     }
     if (role == TimeStringRole) {
-        const auto pendingIt = std::find_if(m_room->pendingEvents().cbegin(), m_room->pendingEvents().cend(), [this](const PendingEventItem &pendingEvent) {
-            return m_event->transactionId() == pendingEvent->transactionId();
+        const auto pendingIt = std::find_if(m_room->pendingEvents().cbegin(), m_room->pendingEvents().cend(), [event](const PendingEventItem &pendingEvent) {
+            return event->transactionId() == pendingEvent->transactionId();
         });
 
         auto lastUpdated = pendingIt == m_room->pendingEvents().cend() ? QDateTime() : pendingIt->lastUpdated();
-        return eventHandler.getTimeString(QStringLiteral("hh:mm"), m_isPending, lastUpdated);
+        return EventHandler::timeString(event, QStringLiteral("hh:mm"), m_isPending, lastUpdated);
     }
     if (role == AuthorRole) {
         return QVariant::fromValue<NeochatRoomMember *>(m_eventSenderObject.get());
     }
     if (role == MediaInfoRole) {
-        return eventHandler.getMediaInfo();
+        return EventHandler::mediaInfo(m_room, event);
     }
     if (role == FileTransferInfoRole) {
-        return QVariant::fromValue(m_room->cachedFileTransferInfo(m_event.get()));
+        return QVariant::fromValue(m_room->cachedFileTransferInfo(event));
     }
     if (role == ItineraryModelRole) {
         return QVariant::fromValue<ItineraryModel *>(m_itineraryModel);
     }
     if (role == LatitudeRole) {
-        return eventHandler.getLatitude();
+        return EventHandler::latitude(event);
     }
     if (role == LongitudeRole) {
-        return eventHandler.getLongitude();
+        return EventHandler::longitude(event);
     }
     if (role == AssetRole) {
-        return eventHandler.getLocationAssetType();
+        return EventHandler::locationAssetType(event);
     }
     if (role == PollHandlerRole) {
         return QVariant::fromValue<PollHandler *>(m_room->poll(m_eventId));
     }
-    if (role == IsReplyRole) {
-        return eventHandler.hasReply();
-    }
     if (role == ReplyEventIdRole) {
-        return eventHandler.getReplyId();
+        return EventHandler::replyId(event);
     }
     if (role == ReplyAuthorRole) {
-        return QVariant::fromValue(eventHandler.getReplyAuthor());
+        return QVariant::fromValue(EventHandler::replyAuthor(m_room, event));
     }
     if (role == ReplyContentModelRole) {
         return QVariant::fromValue<MessageContentModel *>(m_replyModel);
@@ -300,6 +344,12 @@ QVariant MessageContentModel::data(const QModelIndex &index, int role) const
             return QVariant::fromValue<LinkPreviewer *>(emptyLinkPreview);
         }
     }
+    if (role == ChatBarCacheRole) {
+        if (m_room->threadCache()->threadId() == m_eventId) {
+            return QVariant::fromValue<ChatBarCache *>(m_room->threadCache());
+        }
+        return QVariant::fromValue<ChatBarCache *>(m_room->editCache());
+    }
 
     return {};
 }
@@ -327,20 +377,28 @@ QHash<int, QByteArray> MessageContentModel::roleNames() const
     roles[LongitudeRole] = "longitude";
     roles[AssetRole] = "asset";
     roles[PollHandlerRole] = "pollHandler";
-    roles[IsReplyRole] = "isReply";
     roles[ReplyEventIdRole] = "replyEventId";
     roles[ReplyAuthorRole] = "replyAuthor";
     roles[ReplyContentModelRole] = "replyContentModel";
     roles[LinkPreviewerRole] = "linkPreviewer";
+    roles[ChatBarCacheRole] = "chatBarCache";
     return roles;
 }
 
 void MessageContentModel::resetModel()
 {
+    const auto event = m_room->getEvent(m_eventId);
+
     beginResetModel();
     m_components.clear();
 
-    if (m_event == nullptr) {
+    if (m_room->connection()->isIgnored(m_eventSenderId) || m_notFound) {
+        m_components += MessageComponent{MessageComponentType::Text, QString(), {}};
+        endResetModel();
+        return;
+    }
+
+    if (event == nullptr) {
         m_components += MessageComponent{MessageComponentType::Loading, QString(), {}};
         endResetModel();
         return;
@@ -354,16 +412,14 @@ void MessageContentModel::resetModel()
     endResetModel();
 }
 
-void MessageContentModel::resetContent(bool isEditing)
+void MessageContentModel::resetContent(bool isEditing, bool isThreading)
 {
-    Q_ASSERT(m_event != nullptr);
-
     const auto startRow = m_components[0].type == MessageComponentType::Author ? 1 : 0;
     beginRemoveRows({}, startRow, rowCount() - 1);
     m_components.remove(startRow, rowCount() - startRow);
     endRemoveRows();
 
-    const auto newComponents = messageContentComponents(isEditing);
+    const auto newComponents = messageContentComponents(isEditing, isThreading);
     if (newComponents.size() == 0) {
         return;
     }
@@ -372,17 +428,22 @@ void MessageContentModel::resetContent(bool isEditing)
     endInsertRows();
 }
 
-QList<MessageComponent> MessageContentModel::messageContentComponents(bool isEditing)
+QList<MessageComponent> MessageContentModel::messageContentComponents(bool isEditing, bool isThreading)
 {
+    const auto event = m_room->getEvent(m_eventId);
+    if (event == nullptr) {
+        return {};
+    }
+
     QList<MessageComponent> newComponents;
 
-    if (eventCast<const Quotient::RoomMessageEvent>(m_event)
-        && eventCast<const Quotient::RoomMessageEvent>(m_event)->rawMsgtype() == QStringLiteral("m.key.verification.request")) {
+    if (eventCast<const Quotient::RoomMessageEvent>(event)
+        && eventCast<const Quotient::RoomMessageEvent>(event)->rawMsgtype() == QStringLiteral("m.key.verification.request")) {
         newComponents += MessageComponent{MessageComponentType::Verification, QString(), {}};
         return newComponents;
     }
 
-    if (m_event->isRedacted()) {
+    if (event->isRedacted()) {
         newComponents += MessageComponent{MessageComponentType::Text, QString(), {}};
         return newComponents;
     }
@@ -392,37 +453,43 @@ QList<MessageComponent> MessageContentModel::messageContentComponents(bool isEdi
     }
 
     if (isEditing) {
-        newComponents += MessageComponent{MessageComponentType::Edit, QString(), {}};
+        newComponents += MessageComponent{MessageComponentType::ChatBar, QString(), {}};
     } else {
-        EventHandler eventHandler(m_room, m_event.get());
-        newComponents.append(componentsForType(eventHandler.messageComponentType()));
+        newComponents.append(componentsForType(MessageComponentType::typeForEvent(*event)));
     }
 
     if (m_room->urlPreviewEnabled()) {
         newComponents = addLinkPreviews(newComponents);
     }
 
+    // If the event is already threaded the ThreadModel will handle displaying a chat bar.
+    if (isThreading && !EventHandler::isThreaded(event)) {
+        newComponents += MessageComponent{MessageComponentType::ChatBar, QString(), {}};
+    }
+
     return newComponents;
 }
 
 void MessageContentModel::updateReplyModel()
 {
-    if (m_event == nullptr || m_replyModel != nullptr || m_isReply) {
+    const auto event = m_room->getEvent(m_eventId);
+    if (event == nullptr || m_isReply) {
         return;
     }
 
-    EventHandler eventHandler(m_room, m_event.get());
-    if (!eventHandler.hasReply()) {
+    if (!EventHandler::hasReply(event) || (EventHandler::isThreaded(event) && NeoChatConfig::self()->threads())) {
+        if (m_replyModel) {
+            delete m_replyModel;
+        }
         return;
     }
 
-    const auto replyEvent = m_room->findInTimeline(eventHandler.getReplyId());
-    if (replyEvent == m_room->historyEdge()) {
-        m_replyModel = new MessageContentModel(m_room, eventHandler.getReplyId(), true);
-    } else {
-        m_replyModel = new MessageContentModel(m_room, replyEvent->get(), true);
+    if (m_replyModel != nullptr) {
+        return;
     }
 
+    m_replyModel = new MessageContentModel(m_room, EventHandler::replyId(event), true, false, this);
+
     connect(m_replyModel, &MessageContentModel::eventUpdated, this, [this]() {
         Q_EMIT dataChanged(index(0), index(0), {ReplyAuthorRole});
     });
@@ -430,37 +497,37 @@ void MessageContentModel::updateReplyModel()
 
 QList<MessageComponent> MessageContentModel::componentsForType(MessageComponentType::Type type)
 {
+    const auto event = m_room->getEvent(m_eventId);
+    if (event == nullptr) {
+        return {};
+    }
+
     switch (type) {
     case MessageComponentType::Text: {
-        const auto event = eventCast<const Quotient::RoomMessageEvent>(m_event);
-        auto body = EventHandler::rawMessageBody(*event);
-        return TextHandler().textComponents(body, EventHandler::messageBodyInputFormat(*event), m_room, event, event->isReplaced());
+        const auto roomMessageEvent = eventCast<const Quotient::RoomMessageEvent>(event);
+        auto body = EventHandler::rawMessageBody(*roomMessageEvent);
+        return TextHandler().textComponents(body,
+                                            EventHandler::messageBodyInputFormat(*roomMessageEvent),
+                                            m_room,
+                                            roomMessageEvent,
+                                            roomMessageEvent->isReplaced());
     }
     case MessageComponentType::File: {
         QList<MessageComponent> components;
         components += MessageComponent{MessageComponentType::File, QString(), {}};
-        const auto event = eventCast<const Quotient::RoomMessageEvent>(m_event);
+        const auto roomMessageEvent = eventCast<const Quotient::RoomMessageEvent>(event);
 
         if (m_emptyItinerary) {
             if (!m_isReply) {
-                auto fileTransferInfo = m_room->cachedFileTransferInfo(m_event.get());
+                auto fileTransferInfo = m_room->cachedFileTransferInfo(event);
 
 #ifndef Q_OS_ANDROID
-#if Quotient_VERSION_MINOR > 8
-                Q_ASSERT(event->content() != nullptr && event->has<EventContent::FileContent>());
-                const QMimeType mimeType = event->get<EventContent::FileContent>()->mimeType;
-#else
-                Q_ASSERT(event->content() != nullptr && event->hasFileContent());
-                const QMimeType mimeType = event->content()->fileInfo()->mimeType;
-#endif
+                Q_ASSERT(roomMessageEvent->content() != nullptr && roomMessageEvent->has<EventContent::FileContent>());
+                const QMimeType mimeType = roomMessageEvent->get<EventContent::FileContent>()->mimeType;
                 if (mimeType.name() == QStringLiteral("text/plain") || mimeType.parentMimeTypes().contains(QStringLiteral("text/plain"))) {
-#if Quotient_VERSION_MINOR > 8
-                    QString originalName = event->get<EventContent::FileContent>()->originalName;
-#else
-                    QString originalName = event->content()->fileInfo()->originalName;
-#endif
+                    QString originalName = roomMessageEvent->get<EventContent::FileContent>()->originalName;
                     if (originalName.isEmpty()) {
-                        originalName = event->plainBody();
+                        originalName = roomMessageEvent->plainBody();
                     }
                     KSyntaxHighlighting::Repository repository;
                     KSyntaxHighlighting::Definition definitionForFile = repository.definitionForFileName(originalName);
@@ -489,19 +556,27 @@ QList<MessageComponent> MessageContentModel::componentsForType(MessageComponentT
         } else {
             updateItineraryModel();
         }
-        auto body = EventHandler::rawMessageBody(*event);
-        components += TextHandler().textComponents(body, EventHandler::messageBodyInputFormat(*event), m_room, event, event->isReplaced());
+        auto body = EventHandler::rawMessageBody(*roomMessageEvent);
+        components += TextHandler().textComponents(body,
+                                                   EventHandler::messageBodyInputFormat(*roomMessageEvent),
+                                                   m_room,
+                                                   roomMessageEvent,
+                                                   roomMessageEvent->isReplaced());
         return components;
     }
     case MessageComponentType::Image:
     case MessageComponentType::Audio:
     case MessageComponentType::Video: {
-        if (!m_event->is<StickerEvent>()) {
-            const auto event = eventCast<const Quotient::RoomMessageEvent>(m_event);
+        if (!event->is<StickerEvent>()) {
+            const auto roomMessageEvent = eventCast<const Quotient::RoomMessageEvent>(event);
             QList<MessageComponent> components;
             components += MessageComponent{type, QString(), {}};
-            auto body = EventHandler::rawMessageBody(*event);
-            components += TextHandler().textComponents(body, EventHandler::messageBodyInputFormat(*event), m_room, event, event->isReplaced());
+            auto body = EventHandler::rawMessageBody(*roomMessageEvent);
+            components += TextHandler().textComponents(body,
+                                                       EventHandler::messageBodyInputFormat(*roomMessageEvent),
+                                                       m_room,
+                                                       roomMessageEvent,
+                                                       roomMessageEvent->isReplaced());
             return components;
         }
     }
@@ -560,7 +635,7 @@ QList<MessageComponent> MessageContentModel::addLinkPreviews(QList<MessageCompon
 
 void MessageContentModel::closeLinkPreview(int row)
 {
-    if (row < 0 || row > m_components.size()) {
+    if (row < 0 || row >= m_components.size()) {
         qWarning() << "closeLinkPreview() called with row" << row << "which does not exist. m_components.size() =" << m_components.size();
         return;
     }
@@ -570,22 +645,20 @@ void MessageContentModel::closeLinkPreview(int row)
         m_removedLinkPreviews += m_components[row].attributes["link"_ls].toUrl();
         m_components.remove(row);
         m_components.squeeze();
+        endResetModel();
         resetContent();
     }
 }
 
 void MessageContentModel::updateItineraryModel()
 {
-    if (m_room == nullptr || m_event == nullptr) {
+    const auto event = m_room->getEvent(m_eventId);
+    if (m_room == nullptr || event == nullptr) {
         return;
     }
 
-    if (auto event = eventCast<const Quotient::RoomMessageEvent>(m_event)) {
-#if Quotient_VERSION_MINOR > 8
-        if (event->has<EventContent::FileContent>()) {
-#else
-        if (event->hasFileContent()) {
-#endif
+    if (auto roomMessageEvent = eventCast<const Quotient::RoomMessageEvent>(event)) {
+        if (roomMessageEvent->has<EventContent::FileContent>()) {
             auto filePath = m_room->cachedFileTransferInfo(event).localPath;
             if (filePath.isEmpty() && m_itineraryModel != nullptr) {
                 delete m_itineraryModel;
