@@ -95,7 +95,6 @@ void RoomTreeModel::newRoom(Room *r)
     parentItem->insertChild(std::make_unique<RoomTreeItem>(room, parentItem));
     connectRoomSignals(room);
     endInsertRows();
-    qWarning() << "adding room" << type << "new count" << parentItem->childCount();
 }
 
 void RoomTreeModel::leftRoom(Room *r)
@@ -287,6 +286,7 @@ QHash<int, QByteArray> RoomTreeModel::roleNames() const
     roles[IconRole] = "icon";
     roles[AttentionRole] = "attention";
     roles[FavouriteRole] = "favourite";
+    roles[RoomTypeRole] = "roomType";
     return roles;
 }
 
@@ -357,8 +357,7 @@ QVariant RoomTreeModel::data(const QModelIndex &index, int role) const
         if (room->lastEvent() == nullptr || room->lastEventIsSpoiler()) {
             return QString();
         }
-        EventHandler eventHandler(room, room->lastEvent());
-        return eventHandler.subtitleText();
+        return EventHandler::subtitleText(room, room->lastEvent());
     }
     if (role == AvatarImageRole) {
         return room->avatar(128);
@@ -387,6 +386,11 @@ QVariant RoomTreeModel::data(const QModelIndex &index, int role) const
     if (role == FavouriteRole) {
         return room->isFavourite();
     }
+    if (role == RoomTypeRole) {
+        if (room->creation()) {
+            return room->creation()->contentPart<QString>("type"_L1);
+        }
+    }
 
     return {};
 }
