@@ -37,19 +37,15 @@
 
 #include "chatbarcache.h"
 #include "clipboard.h"
-#include "controller.h"
 #include "eventhandler.h"
 #include "events/joinrulesevent.h"
 #include "events/pollevent.h"
 #include "filetransferpseudojob.h"
-#include "jobs/neochatgetcommonroomsjob.h"
 #include "neochatconfig.h"
-#include "notificationsmanager.h"
 #include "roomlastmessageprovider.h"
 #include "spacehierarchycache.h"
 #include "texthandler.h"
 #include "urlhelper.h"
-#include "utils.h"
 
 #ifndef Q_OS_ANDROID
 #include <KIO/Job>
@@ -65,6 +61,7 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
 {
     m_mainCache = new ChatBarCache(this);
     m_editCache = new ChatBarCache(this);
+    m_threadCache = new ChatBarCache(this);
 
     connect(connection, &Connection::accountDataChanged, this, &NeoChatRoom::updatePushNotificationState);
     connect(this, &Room::fileTransferCompleted, this, [this] {
@@ -77,13 +74,8 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
             const auto m_event = evtIt->viewAs<RoomEvent>();
             QString mxcUrl;
             if (auto event = eventCast<const Quotient::RoomMessageEvent>(m_event)) {
-#if Quotient_VERSION_MINOR > 8
                 if (event->has<EventContent::FileContentBase>()) {
                     mxcUrl = event->get<EventContent::FileContentBase>()->url().toString();
-#else
-                if (event->hasFileContent()) {
-                    mxcUrl = event->content()->fileInfo()->url().toString();
-#endif
                 }
             } else if (auto event = eventCast<const Quotient::StickerEvent>(m_event)) {
                 mxcUrl = event->image().url().toString();
@@ -132,45 +124,8 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
             updatePushNotificationState(QStringLiteral("m.push_rules"));
 
             Q_EMIT canEncryptRoomChanged();
-            if (this->joinState() != JoinState::Invite) {
-                return;
-            }
-            auto roomMemberEvent = currentState().get<RoomMemberEvent>(localMember().id());
-
-            auto showNotification = [this, roomMemberEvent] {
-                QImage avatar_image;
-                if (roomMemberEvent && !member(roomMemberEvent->senderId()).avatarUrl().isEmpty()) {
-#if Quotient_VERSION_MINOR > 8
-                    avatar_image = member(roomMemberEvent->senderId()).avatar(128, 128, {});
-#else
-                    avatar_image = memberAvatar(roomMemberEvent->senderId()).get(this->connection(), 128, [] {});
-#endif
-                } else {
-                    qWarning() << "using this room's avatar";
-                    avatar_image = avatar(128);
-                }
-
-                NotificationsManager::instance().postInviteNotification(this,
-                                                                        displayName(),
-                                                                        member(roomMemberEvent->senderId()).htmlSafeDisplayName(),
-                                                                        avatar_image);
-            };
-
-            if (NeoChatConfig::rejectUnknownInvites()) {
-                auto job = this->connection()->callApi<NeochatGetCommonRoomsJob>(roomMemberEvent->senderId());
-                connect(job, &BaseJob::result, this, [this, job, roomMemberEvent, showNotification] {
-                    QJsonObject replyData = job->jsonData();
-                    if (replyData.contains(QStringLiteral("joined"))) {
-                        const bool inAnyOfOurRooms = !replyData[QStringLiteral("joined")].toArray().isEmpty();
-                        if (inAnyOfOurRooms) {
-                            showNotification();
-                        } else {
-                            leaveRoom();
-                        }
-                    }
-                });
-            } else {
-                showNotification();
+            if (this->joinState() == JoinState::Invite) {
+                Q_EMIT showInviteNotification(this);
             }
         },
         Qt::SingleShotConnection);
@@ -261,11 +216,7 @@ QCoro::Task<void> NeoChatRoom::doUploadFile(QUrl url, QString body)
     auto mime = QMimeDatabase().mimeTypeForUrl(url);
     url.setScheme("file"_ls);
     QFileInfo fileInfo(url.isLocalFile() ? url.toLocalFile() : url.toString());
-#if Quotient_VERSION_MINOR > 8
     EventContent::FileContentBase *content;
-#else
-    EventContent::TypedBase *content;
-#endif
     if (mime.name().startsWith("image/"_ls)) {
         QImage image(url.toLocalFile());
         content = new EventContent::ImageContent(url, fileInfo.size(), mime, image.size(), fileInfo.fileName());
@@ -280,11 +231,7 @@ QCoro::Task<void> NeoChatRoom::doUploadFile(QUrl url, QString body)
     } else {
         content = new EventContent::FileContent(url, fileInfo.size(), mime, fileInfo.fileName());
     }
-#if Quotient_VERSION_MINOR > 8
     QString txnId = postFile(body.isEmpty() ? url.fileName() : body, std::unique_ptr<EventContent::FileContentBase>(content));
-#else
-    QString txnId = postFile(body.isEmpty() ? url.fileName() : body, content);
-#endif
     setHasFileUploading(true);
     connect(this, &Room::fileTransferCompleted, [this, txnId](const QString &id, FileSourceInfo) {
         if (id == txnId) {
@@ -320,10 +267,16 @@ void NeoChatRoom::acceptInvitation()
 
 void NeoChatRoom::forget()
 {
-    if (const auto &predecessor = dynamic_cast<NeoChatRoom *>(this->predecessor(JoinState::Join))) {
-        predecessor->forget();
+    QStringList roomIds{id()};
+
+    NeoChatRoom *predecessor = this;
+    while (predecessor = dynamic_cast<NeoChatRoom *>(predecessor->predecessor(JoinState::Join)), predecessor && !roomIds.contains(predecessor->id())) {
+        roomIds += predecessor->id();
+    }
+
+    for (const auto &id : roomIds) {
+        connection()->forgetRoom(id);
     }
-    connection()->forgetRoom(id());
 }
 
 void NeoChatRoom::sendTypingNotification(bool isTyping)
@@ -410,13 +363,8 @@ bool NeoChatRoom::lastEventIsSpoiler() const
 {
     if (auto event = lastEvent()) {
         if (auto e = eventCast<const RoomMessageEvent>(event)) {
-#if Quotient_VERSION_MINOR > 8
             if (e->has<EventContent::TextContent>() && e->content() && e->mimeType().name() == "text/html"_ls) {
                 auto htmlBody = e->get<EventContent::TextContent>()->body;
-#else
-            if (e->hasTextContent() && e->content() && e->mimeType().name() == "text/html"_ls) {
-                auto htmlBody = static_cast<const Quotient::EventContent::TextContent *>(e->content())->body;
-#endif
                 return htmlBody.contains("data-mx-spoiler"_ls);
             }
         }
@@ -539,9 +487,10 @@ void NeoChatRoom::postMessage(const QString &rawText,
                               MessageEventType type,
                               const QString &replyEventId,
                               const QString &relateToEventId,
-                              const QString &threadRootId)
+                              const QString &threadRootId,
+                              const QString &fallbackId)
 {
-    postHtmlMessage(rawText, text, type, replyEventId, relateToEventId, threadRootId);
+    postHtmlMessage(rawText, text, type, replyEventId, relateToEventId, threadRootId, fallbackId);
 }
 
 void NeoChatRoom::postHtmlMessage(const QString &text,
@@ -549,7 +498,8 @@ void NeoChatRoom::postHtmlMessage(const QString &text,
                                   MessageEventType type,
                                   const QString &replyEventId,
                                   const QString &relateToEventId,
-                                  const QString &threadRootId)
+                                  const QString &threadRootId,
+                                  const QString &fallbackId)
 {
     bool isReply = !replyEventId.isEmpty();
     bool isEdit = !relateToEventId.isEmpty();
@@ -560,9 +510,19 @@ void NeoChatRoom::postHtmlMessage(const QString &text,
     }
 
     if (isThread) {
-        EventHandler eventHandler(this, &**replyIt);
+        bool isFallingBack = !fallbackId.isEmpty();
+        QString replyEventId = isFallingBack ? fallbackId : QString();
+        if (isReply) {
+            isFallingBack = false;
+            replyEventId = EventHandler::id(replyIt->get());
+        }
 
-        const bool isFallingBack = !eventHandler.isThreaded();
+        // If we are not replying and there is no fallback ID it means a new thread
+        // is being created.
+        if (!isFallingBack && !isReply) {
+            isFallingBack = true;
+            replyEventId = threadRootId;
+        }
 
         // clang-format off
         QJsonObject json{
@@ -607,12 +567,10 @@ void NeoChatRoom::postHtmlMessage(const QString &text,
     if (isReply) {
         const auto &replyEvt = **replyIt;
 
-        EventHandler eventHandler(this, &**replyIt);
-
         // clang-format off
         QJsonObject json{
           {"msgtype"_ls, msgTypeToString(type)},
-          {"body"_ls, "> <%1> %2\n\n%3"_ls.arg(replyEvt.senderId(), eventHandler.getPlainBody(), text)},
+          {"body"_ls, "> <%1> %2\n\n%3"_ls.arg(replyEvt.senderId(), EventHandler::plainBody(this, &replyEvt), text)},
           {"format"_ls, "org.matrix.custom.html"_ls},
           {"m.relates_to"_ls,
             QJsonObject {
@@ -624,7 +582,7 @@ void NeoChatRoom::postHtmlMessage(const QString &text,
             }
           },
           {"formatted_body"_ls,
-              "<mx-reply><blockquote><a href=\"https://matrix.to/#/%1/%2\">In reply to</a> <a href=\"https://matrix.to/#/%3\">%4</a><br>%5</blockquote></mx-reply>%6"_ls.arg(id(), replyEventId, replyEvt.senderId(), replyEvt.senderId(), eventHandler.getRichBody(), html)
+              "<mx-reply><blockquote><a href=\"https://matrix.to/#/%1/%2\">In reply to</a> <a href=\"https://matrix.to/#/%3\">%4</a><br>%5</blockquote></mx-reply>%6"_ls.arg(id(), replyEventId, replyEvt.senderId(), replyEvt.senderId(), EventHandler::richBody(this, &replyEvt), html)
           }
         };
         // clang-format on
@@ -938,11 +896,7 @@ QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user, QStri
     }
     for (const auto &e : events) {
         auto job = connection()->callApi<RedactEventJob>(id(), QString::fromLatin1(QUrl::toPercentEncoding(e)), connection()->generateTxnId(), reason);
-#if Quotient_VERSION_MINOR > 8
         co_await qCoro(job.get(), &BaseJob::finished);
-#else
-        co_await qCoro(job, &BaseJob::finished);
-#endif
         if (job->error() != BaseJob::Success) {
             qWarning() << "Error: \"" << job->error() << "\" while deleting messages. Aborting";
             break;
@@ -950,11 +904,6 @@ QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user, QStri
     }
 }
 
-void NeoChatRoom::clearInvitationNotification()
-{
-    NotificationsManager::instance().clearInvitationNotification(id());
-}
-
 bool NeoChatRoom::hasParent() const
 {
     return currentState().eventsOfType("m.space.parent"_ls).size() > 0;
@@ -1246,11 +1195,7 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
         for (const auto &i : roomRuleArray) {
             QJsonObject roomRule = i.toObject();
             if (roomRule["rule_id"_ls] == id()) {
-#if Quotient_VERSION_MINOR > 8
                 connection()->callApi<DeletePushRuleJob>("room"_ls, id());
-#else
-                connection()->callApi<DeletePushRuleJob>(QLatin1String("global"), "room"_ls, id());
-#endif
             }
         }
     }
@@ -1261,11 +1206,7 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
         for (const auto &i : overrideRuleArray) {
             QJsonObject overrideRule = i.toObject();
             if (overrideRule["rule_id"_ls] == id()) {
-#if Quotient_VERSION_MINOR > 8
                 connection()->callApi<DeletePushRuleJob>("override"_ls, id());
-#else
-                connection()->callApi<DeletePushRuleJob>("global"_ls, "override"_ls, id());
-#endif
             }
         }
     }
@@ -1301,17 +1242,9 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
         const QList<PushCondition> conditions = {pushCondition};
 
         // Add new override rule and make sure it's enabled
-#if Quotient_VERSION_MINOR > 8
         auto job = connection()->callApi<SetPushRuleJob>("override"_ls, id(), actions, QString(), QString(), conditions, QString());
-#else
-        auto job = connection()->callApi<SetPushRuleJob>("global"_ls, "override"_ls, id(), actions, QString(), QString(), conditions, QString());
-#endif
         connect(job, &BaseJob::success, this, [this]() {
-#if Quotient_VERSION_MINOR > 8
             auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("override"_ls, id(), true);
-#else
-            auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("global"_ls, "override"_ls, id(), true);
-#endif
             connect(enableJob, &BaseJob::success, this, [this]() {
                 m_pushNotificationStateUpdating = false;
             });
@@ -1335,17 +1268,9 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
         // No conditions for a room rule
         const QList<PushCondition> conditions;
 
-#if Quotient_VERSION_MINOR > 8
         auto setJob = connection()->callApi<SetPushRuleJob>("room"_ls, id(), actions, QString(), QString(), conditions, QString());
-#else
-        auto setJob = connection()->callApi<SetPushRuleJob>("global"_ls, "room"_ls, id(), actions, QString(), QString(), conditions, QString());
-#endif
         connect(setJob, &BaseJob::success, this, [this]() {
-#if Quotient_VERSION_MINOR > 8
             auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("room"_ls, id(), true);
-#else
-            auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("global"_ls, "room"_ls, id(), true);
-#endif
             connect(enableJob, &BaseJob::success, this, [this]() {
                 m_pushNotificationStateUpdating = false;
             });
@@ -1374,17 +1299,9 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
         const QList<PushCondition> conditions;
 
         // Add new room rule and make sure enabled
-#if Quotient_VERSION_MINOR > 8
         auto setJob = connection()->callApi<SetPushRuleJob>("room"_ls, id(), actions, QString(), QString(), conditions, QString());
-#else
-        auto setJob = connection()->callApi<SetPushRuleJob>("global"_ls, "room"_ls, id(), actions, QString(), QString(), conditions, QString());
-#endif
         connect(setJob, &BaseJob::success, this, [this]() {
-#if Quotient_VERSION_MINOR > 8
             auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("room"_ls, id(), true);
-#else
-            auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("global"_ls, "room"_ls, id(), true);
-#endif
             connect(enableJob, &BaseJob::success, this, [this]() {
                 m_pushNotificationStateUpdating = false;
             });
@@ -1453,9 +1370,9 @@ void NeoChatRoom::updatePushNotificationState(QString type)
 void NeoChatRoom::reportEvent(const QString &eventId, const QString &reason)
 {
     auto job = connection()->callApi<ReportContentJob>(id(), eventId, -50, reason);
-    connect(job, &BaseJob::finished, this, [job]() {
+    connect(job, &BaseJob::finished, this, [this, job]() {
         if (job->error() == BaseJob::Success) {
-            Q_EMIT Controller::instance().showMessage(Controller::Positive, i18n("Report sent successfully."));
+            Q_EMIT showMessage(MessageType::Positive, i18n("Report sent successfully."));
         }
     });
 }
@@ -1475,11 +1392,7 @@ void NeoChatRoom::openEventMediaExternally(const QString &eventId)
     const auto evtIt = findInTimeline(eventId);
     if (evtIt != messageEvents().rend() && is<RoomMessageEvent>(**evtIt)) {
         const auto event = evtIt->viewAs<RoomMessageEvent>();
-#if Quotient_VERSION_MINOR > 8
         if (event->has<EventContent::FileContent>()) {
-#else
-        if (event->hasFileContent()) {
-#endif
             const auto transferInfo = cachedFileTransferInfo(event);
             if (transferInfo.completed()) {
                 UrlHelper helper;
@@ -1512,11 +1425,7 @@ void NeoChatRoom::copyEventMedia(const QString &eventId)
     const auto evtIt = findInTimeline(eventId);
     if (evtIt != messageEvents().rend() && is<RoomMessageEvent>(**evtIt)) {
         const auto event = evtIt->viewAs<RoomMessageEvent>();
-#if Quotient_VERSION_MINOR > 8
         if (event->has<EventContent::FileContent>()) {
-#else
-        if (event->hasFileContent()) {
-#endif
             const auto transferInfo = fileTransferInfo(eventId);
             if (transferInfo.completed()) {
                 Clipboard clipboard;
@@ -1549,13 +1458,8 @@ FileTransferInfo NeoChatRoom::cachedFileTransferInfo(const Quotient::RoomEvent *
     QString mxcUrl;
     int total = 0;
     if (auto evt = eventCast<const Quotient::RoomMessageEvent>(event)) {
-#if Quotient_VERSION_MINOR > 8
         if (evt->has<EventContent::FileContent>()) {
             const auto fileContent = evt->get<EventContent::FileContent>();
-#else
-        if (evt->hasFileContent()) {
-            const auto fileContent = evt->content()->fileInfo();
-#endif
 
             mxcUrl = fileContent->url().toString();
             total = fileContent->payloadSize;
@@ -1602,6 +1506,11 @@ ChatBarCache *NeoChatRoom::editCache() const
     return m_editCache;
 }
 
+ChatBarCache *NeoChatRoom::threadCache() const
+{
+    return m_threadCache;
+}
+
 void NeoChatRoom::replyLastMessage()
 {
     const auto &timelineBottom = messageEvents().rbegin();
@@ -1816,12 +1725,16 @@ QByteArray NeoChatRoom::roomAcountDataJson(const QString &eventType)
 void NeoChatRoom::downloadEventFromServer(const QString &eventId)
 {
     if (findInTimeline(eventId) != historyEdge()) {
+        // For whatever reason the event has now appeared so the function that called
+        // this need to whatever it wanted to do with the event.
+        Q_EMIT extraEventLoaded(eventId);
         return;
     }
     auto job = connection()->callApi<GetOneRoomEventJob>(id(), eventId);
     connect(job, &BaseJob::success, this, [this, job, eventId] {
         // The event may have arrived in the meantime so check it's not in the timeline.
         if (findInTimeline(eventId) != historyEdge()) {
+            Q_EMIT extraEventLoaded(eventId);
             return;
         }
 
@@ -1829,6 +1742,11 @@ void NeoChatRoom::downloadEventFromServer(const QString &eventId)
         m_extraEvents.push_back(std::move(event));
         Q_EMIT extraEventLoaded(eventId);
     });
+    connect(job, &BaseJob::failure, this, [this, job, eventId] {
+        if (job->error() == BaseJob::NotFound) {
+            Q_EMIT extraEventNotFound(eventId);
+        }
+    });
 }
 
 const RoomEvent *NeoChatRoom::getEvent(const QString &eventId) const
@@ -1899,13 +1817,4 @@ void NeoChatRoom::setRoomState(const QString &type, const QString &stateKey, con
     setState(type, stateKey, QJsonDocument::fromJson(content).object());
 }
 
-#if Quotient_VERSION_MINOR == 8
-QList<RoomMember> NeoChatRoom::otherMembersTyping() const
-{
-    auto memberTyping = membersTyping();
-    memberTyping.removeAll(localMember());
-    return memberTyping;
-}
-#endif
-
 #include "moc_neochatroom.cpp"
