@@ -12,6 +12,7 @@
 #include <QCoroTask>
 #include <Quotient/roommember.h>
 
+#include "enums/messagetype.h"
 #include "enums/pushrule.h"
 #include "neochatroommember.h"
 #include "pollhandler.h"
@@ -202,9 +203,10 @@ class NeoChatRoom : public Quotient::Room
      */
     Q_PROPERTY(ChatBarCache *editCache READ editCache CONSTANT)
 
-#if Quotient_VERSION_MINOR == 8
-    Q_PROPERTY(QList<Quotient::RoomMember> otherMembersTyping READ otherMembersTyping NOTIFY typingChanged)
-#endif
+    /**
+     * @brief The cache for the thread chat bar in the room.
+     */
+    Q_PROPERTY(ChatBarCache *threadCache READ threadCache CONSTANT)
 
 public:
     explicit NeoChatRoom(Quotient::Connection *connection, QString roomId, Quotient::JoinState joinState = {});
@@ -412,8 +414,6 @@ public:
 
     bool readOnly() const;
 
-    Q_INVOKABLE void clearInvitationNotification();
-
     [[nodiscard]] QString joinRule() const;
 
     /**
@@ -512,6 +512,8 @@ public:
 
     ChatBarCache *editCache() const;
 
+    ChatBarCache *threadCache() const;
+
     /**
      * @brief Reply to the last message sent in the timeline.
      *
@@ -610,16 +612,13 @@ private:
 
     ChatBarCache *m_mainCache;
     ChatBarCache *m_editCache;
+    ChatBarCache *m_threadCache;
 
     QCache<QString, PollHandler> m_polls;
     std::vector<Quotient::event_ptr_tt<Quotient::RoomEvent>> m_extraEvents;
     void cleanupExtraEventRange(Quotient::RoomEventsRange events);
     void cleanupExtraEvent(const QString &eventId);
 
-#if Quotient_VERSION_MINOR == 8
-    QList<Quotient::RoomMember> otherMembersTyping() const;
-#endif
-
 private Q_SLOTS:
     void updatePushNotificationState(QString type);
 
@@ -648,6 +647,20 @@ Q_SIGNALS:
     void urlPreviewEnabledChanged();
     void maxRoomVersionChanged();
     void extraEventLoaded(const QString &eventId);
+    void extraEventNotFound(const QString &eventId);
+
+    /**
+     * @brief Request a message be shown to the user of the given type.
+     */
+    void showMessage(MessageType::Type messageType, const QString &message);
+
+    /**
+     * @brief Request a notification be shown for an invite to this room.
+     *
+     * @note This may later be blocked if there are any rules on where invites can
+     * come from, but this is not NeoChatRoom's responsibility.
+     */
+    void showInviteNotification(NeoChatRoom *room);
 
 public Q_SLOTS:
     /**
@@ -692,7 +705,8 @@ public Q_SLOTS:
                      Quotient::MessageEventType type = Quotient::MessageEventType::Text,
                      const QString &replyEventId = QString(),
                      const QString &relateToEventId = QString(),
-                     const QString &threadRootId = QString());
+                     const QString &threadRootId = QString(),
+                     const QString &fallbackId = QString());
 
     /**
      * @brief Send an html message to the room.
@@ -708,7 +722,8 @@ public Q_SLOTS:
                          Quotient::MessageEventType type = Quotient::MessageEventType::Text,
                          const QString &replyEventId = QString(),
                          const QString &relateToEventId = QString(),
-                         const QString &threadRootId = QString());
+                         const QString &threadRootId = QString(),
+                         const QString &fallbackId = QString());
 
     /**
      * @brief Set the room avatar.
