@@ -9,11 +9,13 @@
 
 #include <KLocalizedString>
 #include <KNotification>
+#include <KNotificationPermission>
 #include <KNotificationReplyAction>
 
 #include <QPainter>
 #include <Quotient/accountregistry.h>
 #include <Quotient/csapi/pushrules.h>
+#include <Quotient/events/roommemberevent.h>
 #include <Quotient/user.h>
 
 #ifdef HAVE_KIO
@@ -21,6 +23,8 @@
 #endif
 
 #include "controller.h"
+#include "jobs/neochatgetcommonroomsjob.h"
+#include "neochatconfig.h"
 #include "neochatconnection.h"
 #include "neochatroom.h"
 #include "roommanager.h"
@@ -29,12 +33,6 @@
 
 using namespace Quotient;
 
-NotificationsManager &NotificationsManager::instance()
-{
-    static NotificationsManager _instance;
-    return _instance;
-}
-
 NotificationsManager::NotificationsManager(QObject *parent)
     : QObject(parent)
 {
@@ -42,6 +40,25 @@ NotificationsManager::NotificationsManager(QObject *parent)
 
 void NotificationsManager::handleNotifications(QPointer<NeoChatConnection> connection)
 {
+    if (KNotificationPermission::checkPermission() == Qt::PermissionStatus::Granted) {
+        startNotificationJob(connection);
+    } else if (!permissionAsked) {
+        KNotificationPermission::requestPermission(this, [this, connection](Qt::PermissionStatus result) {
+            if (result == Qt::PermissionStatus::Granted) {
+                startNotificationJob(connection);
+            } else {
+                permissionAsked = true;
+            }
+        });
+    }
+}
+
+void NotificationsManager::startNotificationJob(QPointer<NeoChatConnection> connection)
+{
+    if (connection == nullptr) {
+        return;
+    }
+
     if (!m_connActiveJob.contains(connection->user()->id())) {
         auto job = connection->callApi<GetNotificationsJob>();
         m_connActiveJob.append(connection->user()->id());
@@ -131,11 +148,7 @@ void NotificationsManager::processNotificationJob(QPointer<NeoChatConnection> co
 
         QImage avatar_image;
         if (!sender.avatarUrl().isEmpty()) {
-#if Quotient_VERSION_MINOR > 8
             avatar_image = room->member(sender.id()).avatar(128, 128, {});
-#else
-            avatar_image = room->memberAvatar(sender.id()).get(connection, 128, {});
-#endif
         } else {
             avatar_image = room->avatar(128);
         }
@@ -244,15 +257,53 @@ void NotificationsManager::postNotification(NeoChatRoom *room,
     notification->sendEvent();
 }
 
-void NotificationsManager::postInviteNotification(NeoChatRoom *rawRoom, const QString &title, const QString &sender, const QImage &icon)
+void NotificationsManager::postInviteNotification(NeoChatRoom *rawRoom)
 {
     QPointer room(rawRoom);
-    QPixmap img;
-    img.convertFromImage(icon);
+
+    const auto roomMemberEvent = room->currentState().get<RoomMemberEvent>(room->localMember().id());
+    if (roomMemberEvent == nullptr) {
+        return;
+    }
+
+    if (NeoChatConfig::rejectUnknownInvites()) {
+        auto job = room->connection()->callApi<NeochatGetCommonRoomsJob>(roomMemberEvent->senderId());
+        connect(job, &BaseJob::result, this, [this, job, room] {
+            QJsonObject replyData = job->jsonData();
+            if (replyData.contains(QStringLiteral("joined"))) {
+                const bool inAnyOfOurRooms = !replyData[QStringLiteral("joined")].toArray().isEmpty();
+                if (inAnyOfOurRooms) {
+                    doPostInviteNotification(room);
+                } else {
+                    room->leaveRoom();
+                }
+            }
+        });
+    } else {
+        doPostInviteNotification(room);
+    }
+}
+
+void NotificationsManager::doPostInviteNotification(QPointer<NeoChatRoom> room)
+{
+    const auto roomMemberEvent = room->currentState().get<RoomMemberEvent>(room->localMember().id());
+    if (roomMemberEvent == nullptr) {
+        return;
+    }
+    const auto sender = room->member(roomMemberEvent->senderId());
+
+    QImage avatar_image;
+    if (roomMemberEvent && !room->member(roomMemberEvent->senderId()).avatarUrl().isEmpty()) {
+        avatar_image = room->member(roomMemberEvent->senderId()).avatar(128, 128, {});
+    } else {
+        qWarning() << "using this room's avatar";
+        avatar_image = room->avatar(128);
+    }
+
     KNotification *notification = new KNotification(QStringLiteral("invite"));
-    notification->setText(i18n("%1 invited you to a room", sender));
-    notification->setTitle(title);
-    notification->setPixmap(createNotificationImage(icon, nullptr));
+    notification->setText(i18n("%1 invited you to a room", sender.htmlSafeDisplayName()));
+    notification->setTitle(room->displayName());
+    notification->setPixmap(createNotificationImage(avatar_image, nullptr));
     auto defaultAction = notification->addDefaultAction(i18n("Open this invitation in NeoChat"));
     connect(defaultAction, &KNotificationAction::activated, this, [notification, room]() {
         if (!room) {
