@@ -7,16 +7,13 @@
 #include "chatbarcache.h"
 #include "controller.h"
 #include "eventhandler.h"
-#include "models/timelinemodel.h"
 #include "neochatconfig.h"
 #include "neochatconnection.h"
 #include "neochatroom.h"
-#include "neochatroommember.h"
 #include "spacehierarchycache.h"
 #include "urlhelper.h"
 
 #include <KLocalizedString>
-#include <KRuntimePlatform>
 #include <QDesktopServices>
 #include <QQuickTextDocument>
 #include <QStandardPaths>
@@ -43,6 +40,16 @@ RoomManager::RoomManager(QObject *parent)
     , m_mediaMessageFilterModel(new MediaMessageFilterModel(this, m_messageFilterModel))
     , m_userListModel(new UserListModel(this))
 {
+#if defined(Q_OS_ANDROID) || defined(Q_OS_IOS) || defined(UBUNTU_TOUCH)
+    m_isMobile = true;
+#else
+    // Mostly for debug purposes and for platforms which are always mobile,
+    // such as Plasma Mobile
+    if (qEnvironmentVariableIsSet("QT_QUICK_CONTROLS_MOBILE")) {
+        m_isMobile = QByteArrayList{"1", "true"}.contains(qgetenv("QT_QUICK_CONTROLS_MOBILE"));
+    }
+#endif
+
     m_lastRoomConfig = m_config->group(QStringLiteral("LastOpenRoom"));
     m_lastSpaceConfig = m_config->group(QStringLiteral("LastOpenSpace"));
     m_directChatsConfig = m_config->group(QStringLiteral("DirectChatsActive"));
@@ -136,7 +143,7 @@ void RoomManager::resolveResource(const QString &idOrUri, const QString &action)
 {
     Uri uri{idOrUri};
     if (!uri.isValid()) {
-        Q_EMIT warning(i18n("Malformed or empty Matrix id"), i18n("%1 is not a correct Matrix identifier", idOrUri));
+        Q_EMIT showMessage(MessageType::Warning, i18n("Malformed or empty Matrix id<br />%1 is not a correct Matrix identifier", idOrUri));
         return;
     }
 
@@ -160,21 +167,6 @@ void RoomManager::resolveResource(const QString &idOrUri, const QString &action)
         if ((uri.type() == Uri::RoomAlias || uri.type() == Uri::RoomId) && action != "no_join"_ls) {
             Q_EMIT askJoinRoom(uri.primaryId());
         }
-    } else {
-        if (result != Quotient::UriResolved) {
-            return;
-        }
-
-        if (uri.type() == Uri::RoomAlias || uri.type() == Uri::RoomId) {
-            connect(
-                m_connection.get(),
-                &NeoChatConnection::newRoom,
-                this,
-                [this, uri](Room *room) {
-                    resolveResource(room->id());
-                },
-                Qt::SingleShotConnection);
-        }
     }
 }
 
@@ -207,19 +199,23 @@ void RoomManager::viewEventSource(const QString &eventId)
 void RoomManager::viewEventMenu(const QString &eventId, NeoChatRoom *room, NeochatRoomMember *sender, const QString &selectedText)
 {
     const auto &event = **room->findInTimeline(eventId);
-    const auto eventHandler = EventHandler(room, &event);
 
-    if (eventHandler.getMediaInfo().contains("mimeType"_ls)) {
+    if (EventHandler::mediaInfo(room, &event).contains("mimeType"_ls)) {
         Q_EMIT showFileMenu(eventId,
                             sender,
-                            eventHandler.messageComponentType(),
-                            eventHandler.getPlainBody(),
-                            eventHandler.getMediaInfo()["mimeType"_ls].toString(),
+                            MessageComponentType::typeForEvent(event),
+                            EventHandler::plainBody(room, &event),
+                            EventHandler::mediaInfo(room, &event)["mimeType"_ls].toString(),
                             room->fileTransferInfo(eventId));
         return;
     }
 
-    Q_EMIT showMessageMenu(eventId, sender, eventHandler.messageComponentType(), eventHandler.getPlainBody(), eventHandler.getRichBody(), selectedText);
+    Q_EMIT showMessageMenu(eventId,
+                           sender,
+                           MessageComponentType::typeForEvent(event),
+                           EventHandler::plainBody(room, &event),
+                           EventHandler::richBody(room, &event),
+                           selectedText);
 }
 
 bool RoomManager::hasOpenRoom() const
@@ -240,8 +236,9 @@ void RoomManager::loadInitialRoom()
         resolveResource(m_arg);
     }
 
-    const auto runtimePlatform = KRuntimePlatform::runtimePlatform();
-    if (runtimePlatform.contains(QStringLiteral("phone")) || runtimePlatform.contains(QStringLiteral("handset"))) {
+    if (m_isMobile) {
+        // We still need to remember the last space on mobile
+        setCurrentSpace(m_lastSpaceConfig.readEntry(m_connection->userId(), QString()), false);
         // We don't want to open a room on startup on mobile
         return;
     }
@@ -316,7 +313,8 @@ void RoomManager::visitRoom(Room *r, const QString &eventId)
         return;
     }
 
-    if (m_currentRoom && m_currentRoom->id() == room->id()) {
+    // It's important that we compare room *objects* here, not just room *ids*, since we need to deal with the object changing when going invite -> joined
+    if (m_currentRoom && m_currentRoom == room) {
         Q_EMIT goToEvent(eventId);
     } else {
         setCurrentRoom(room->id());
@@ -326,19 +324,11 @@ void RoomManager::visitRoom(Room *r, const QString &eventId)
 void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAliasOrId, const QStringList &viaServers)
 {
     auto job = account->joinRoom(roomAliasOrId, viaServers);
-#if Quotient_VERSION_MINOR > 8
     connect(
         job.get(),
         &Quotient::BaseJob::finished,
         this,
         [this, account](Quotient::BaseJob *finish) {
-#else
-    connect(
-        job,
-        &Quotient::BaseJob::finished,
-        this,
-        [this, account](Quotient::BaseJob *finish) {
-#endif
             if (finish->status() == Quotient::BaseJob::Success) {
                 connect(
                     account,
@@ -349,7 +339,7 @@ void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAli
                     },
                     Qt::SingleShotConnection);
             } else {
-                Q_EMIT warning(i18n("Failed to join room"), finish->errorString());
+                Q_EMIT showMessage(MessageType::Warning, i18n("Failed to join room<br />%1", finish->errorString()));
             }
         },
         Qt::SingleShotConnection);
@@ -357,39 +347,27 @@ void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAli
 
 void RoomManager::knockRoom(NeoChatConnection *account, const QString &roomAliasOrId, const QString &reason, const QStringList &viaServers)
 {
-#if Quotient_VERSION_MINOR > 8
     auto job = account->callApi<KnockRoomJob>(roomAliasOrId, viaServers, viaServers, reason);
-#else
-    auto job = account->callApi<KnockRoomJob>(roomAliasOrId, viaServers, reason);
-#endif
     // Upon completion, ensure a room object is created in case it hasn't come
     // with a sync yet. If the room object is not there, provideRoom() will
     // create it in Join state. finished() is used here instead of success()
     // to overtake clients that may add their own slots to finished().
-#if Quotient_VERSION_MINOR > 8
     connect(
         job.get(),
         &BaseJob::finished,
         this,
         [this, job, account] {
-#else
-    connect(
-        job,
-        &BaseJob::finished,
-        this,
-        [this, job, account] {
-#endif
             if (job->status() == Quotient::BaseJob::Success) {
                 connect(
                     account,
                     &NeoChatConnection::newRoom,
                     this,
                     [this](Quotient::Room *room) {
-                        Q_EMIT Controller::instance().showMessage(Controller::Info, i18n("You requested to join '%1'", room -> name()));
+                        Q_EMIT showMessage(MessageType::Information, i18n("You requested to join '%1'", room->name()));
                     },
                     Qt::SingleShotConnection);
             } else {
-                Q_EMIT warning(i18n("Failed to request joining room"), job->errorString());
+                Q_EMIT showMessage(MessageType::Warning, i18n("Failed to request joining room<br />%1", job->errorString()));
             }
         },
         Qt::SingleShotConnection);
@@ -435,7 +413,28 @@ void RoomManager::setConnection(NeoChatConnection *connection)
     if (m_connection == connection) {
         return;
     }
+
+    if (m_connection != nullptr) {
+        m_connection->disconnect(this);
+    }
+
     m_connection = connection;
+
+    if (m_connection != nullptr) {
+        connect(m_connection, &NeoChatConnection::showMessage, this, &RoomManager::showMessage);
+        connect(m_connection, &NeoChatConnection::createdRoom, this, [this](Quotient::Room *room) {
+            resolveResource(room->id());
+        });
+        connect(m_connection, &NeoChatConnection::directChatAvailable, this, [this](Quotient::Room *directChat) {
+            resolveResource(directChat->id());
+        });
+        connect(m_connection, &NeoChatConnection::joinedRoom, this, [this](const auto &room, const auto &previousRoom) {
+            if (m_currentRoom == previousRoom) {
+                resolveResource(room->id());
+            }
+        });
+    }
+
     Q_EMIT connectionChanged();
 }
 
@@ -455,20 +454,30 @@ void RoomManager::setCurrentSpace(const QString &spaceId, bool setRoom)
     if (!setRoom) {
         return;
     }
+
     if (spaceId.length() > 3) {
         resolveResource(spaceId, "no_join"_ls);
-    } else {
+    } else if (!m_isMobile) {
         visitRoom({}, {});
     }
 }
 
 void RoomManager::setCurrentRoom(const QString &roomId)
 {
+    if (m_currentRoom != nullptr) {
+        m_currentRoom->disconnect(this);
+    }
+
     if (roomId.isEmpty()) {
         m_currentRoom = nullptr;
     } else {
         m_currentRoom = dynamic_cast<NeoChatRoom *>(m_connection->room(roomId));
     }
+
+    if (m_currentRoom != nullptr) {
+        connect(m_currentRoom, &NeoChatRoom::showMessage, this, &RoomManager::showMessage);
+    }
+
     Q_EMIT currentRoomChanged();
     if (m_connection) {
         m_lastRoomConfig.writeEntry(m_connection->userId(), roomId);
@@ -513,6 +522,11 @@ void RoomManager::setCurrentRoom(const QString &roomId)
     setCurrentSpace({}, false);
 }
 
+void RoomManager::clearCurrentRoom()
+{
+    setCurrentRoom(QString());
+}
+
 QString RoomManager::currentSpace() const
 {
     return m_currentSpaceId;
