@@ -13,7 +13,7 @@
 
 #include "chatdocumenthandler.h"
 #include "enums/messagecomponenttype.h"
-#include "eventhandler.h"
+#include "enums/messagetype.h"
 #include "models/mediamessagefiltermodel.h"
 #include "models/messagefiltermodel.h"
 #include "models/roomlistmodel.h"
@@ -252,6 +252,11 @@ public:
      */
     void setConnection(NeoChatConnection *connection);
 
+    /**
+     * @brief Clear the current room.
+     */
+    Q_INVOKABLE void clearCurrentRoom();
+
     /**
      * Closes the current room and space; for situations, where it is unclear which room should be opened.
      */
@@ -329,9 +334,9 @@ Q_SIGNALS:
     void askDirectChatConfirmation(const Quotient::User *user);
 
     /**
-     * @brief Displays warning to the user.
+     * @brief Request a message be shown to the user of the given type.
      */
-    void warning(const QString &title, const QString &message);
+    void showMessage(MessageType::Type messageType, const QString &message);
 
     void chatDocumentHandlerChanged();
 
@@ -344,6 +349,8 @@ Q_SIGNALS:
     void currentSpaceChanged();
 
 private:
+    bool m_isMobile = false;
+
     void openRoomForActiveConnection();
 
     /** The room currently being shown in the main view (RoomPage.qml). This can be null, if there is no room.
