@@ -17,10 +17,11 @@
 #include <Kirigami/Platform/PlatformTheme>
 
 #include "messagecomponenttype.h"
-#include "messagecontentmodel.h"
 #include "models/customemojimodel.h"
 #include "utils.h"
 
+using namespace Qt::StringLiterals;
+
 static const QStringList allowedTags = {
     QStringLiteral("font"),    QStringLiteral("del"),    QStringLiteral("h1"),         QStringLiteral("h2"),     QStringLiteral("h3"),    QStringLiteral("h4"),
     QStringLiteral("h5"),      QStringLiteral("h6"),     QStringLiteral("blockquote"), QStringLiteral("p"),      QStringLiteral("a"),     QStringLiteral("ul"),
@@ -94,6 +95,12 @@ QString TextHandler::handleSendText()
 
         m_nextTokenType = nextTokenType(m_dataBuffer, m_pos, m_nextToken, m_nextTokenType);
     }
+
+    if (outputString.count("<p>"_L1) == 1 && outputString.count("</p>"_L1) == 1 && outputString.startsWith("<p>"_L1) && outputString.endsWith("</p>"_L1)) {
+        outputString.remove("<p>"_L1);
+        outputString.remove("</p>"_L1);
+    }
+
     return outputString;
 }
 
@@ -171,6 +178,12 @@ TextHandler::handleRecieveRichText(Qt::TextFormat inputFormat, const NeoChatRoom
      * convert as that is what is needed for Qt::RichText.
      */
     outputString.replace(TextRegex::strikethrough, QStringLiteral("<s>\\1</s>"));
+
+    if (outputString.count("<p>"_L1) == 1 && outputString.count("</p>"_L1) == 1 && outputString.startsWith("<p>"_L1) && outputString.endsWith("</p>"_L1)) {
+        outputString.remove("<p>"_L1);
+        outputString.remove("</p>"_L1);
+    }
+
     return outputString;
 }
 
@@ -459,7 +472,7 @@ QString TextHandler::cleanAttributes(const QString &tag, const QString &tagStrin
         nextAttributeIndex += 1;
 
         while (nextAttributeIndex < tagString.length()) {
-            nextSpaceIndex = tagString.indexOf(TextRegex::endTagType, nextAttributeIndex);
+            nextSpaceIndex = tagString.indexOf(TextRegex::endAttributeType, nextAttributeIndex);
             if (nextSpaceIndex == -1) {
                 nextSpaceIndex = tagString.length();
             }
@@ -505,7 +518,7 @@ QVariantMap TextHandler::getAttributes(const QString &tag, const QString &tagStr
         nextAttributeIndex += 1;
 
         while (nextAttributeIndex < tagString.length()) {
-            nextSpaceIndex = tagString.indexOf(TextRegex::endTagType, nextAttributeIndex);
+            nextSpaceIndex = tagString.indexOf(TextRegex::endAttributeType, nextAttributeIndex);
             if (nextSpaceIndex == -1) {
                 nextSpaceIndex = tagString.length();
             }
@@ -614,6 +627,7 @@ QString TextHandler::unescapeHtml(QString stringIn)
     stringIn.replace(QStringLiteral("&gt;"), QStringLiteral(">"));
     stringIn.replace(QStringLiteral("&amp;"), QStringLiteral("&"));
     stringIn.replace(QStringLiteral("&quot;"), QStringLiteral("\""));
+    stringIn.replace(QStringLiteral("&#x27;"), QStringLiteral("'"));
     return stringIn;
 }
 
@@ -625,7 +639,7 @@ QString TextHandler::linkifyUrls(QString stringIn)
         int skip = 0;
         if (match.captured(0).size() > 0) {
             if (stringIn.left(index).count(QStringLiteral("<code>")) == stringIn.left(index).count(QStringLiteral("</code>"))) {
-                auto replacement = QStringLiteral("<a href=\"https://matrix.to/#/%1\">%1</a>").arg(match.captured(2));
+                auto replacement = QStringLiteral("<a href=\"https://matrix.to/#/%1\">%1</a>").arg(match.captured(1));
                 stringIn = stringIn.replace(index, match.captured(0).size(), replacement);
             } else {
                 skip = match.captured().length();
